import os
import requests
from typing import Dict
from base64 import b64encode

FIVETRAN_API_KEY = os.getenv("FIVETRAN_API_KEY")
FIVETRAN_API_SECRET = os.getenv("FIVETRAN_API_SECRET")


class FivetranClient:
    def __init__(self, api_key, api_secret):
        self.session: requests.Session = requests.Session()
        header_token = b64encode(f"{api_key}:{api_secret}".encode("utf-8")).decode(
            "utf-8"
        )
        self.session.headers = {
            "Authorization": f"Basic {header_token}",
            "Content-Type": "application/json",
        }

    def get_connector_details(self, connector_id: str) -> Dict:
        url = f"https://api.fivetran.com/v1/connectors/{connector_id}"
        response = self.session.get(url)
        response.raise_for_status()
        return response.json()

    def get_all_groups(self) -> Dict:
        url = "https://api.fivetran.com/v1/groups"
        response = self.session.get(url, params={"limit": 1000})
        response.raise_for_status()
        result = response.json()
        cursor = result["data"].get("next_cursor")
        while cursor:
            response = self.session.get(url, params={"cursor": cursor, "limit": 1000})
            response.raise_for_status()
            result["data"]["items"].extend(response.json()["data"]["items"])
            cursor = response.json()["data"].get("next_cursor")
        return result

    def get_all_connectors_in_group(self, group_id: str) -> Dict:
        url = f"https://api.fivetran.com/v1/groups/{group_id}/connectors"
        response = self.session.get(url)
        response.raise_for_status()
        return response.json()

    def get_connector_schema_config(self, connector_id: str) -> Dict:
        url = f"https://api.fivetran.com/v1/connectors/{connector_id}/schemas"
        response = self.session.get(url)
        response.raise_for_status()
        return response.json()



from sqlalchemy.engine import create_engine


class Warehouse:
    def __init__(self, credentials):
        self.type = credentials["type"]
        self.database = (
            credentials.get("database")
            or credentials.get("project")
            or credentials.get("dbname")
        )
        self.schema = credentials.get("schema") or credentials.get("dataset")
        self.user = credentials.get("user")
        self.password = credentials.get("password") or credentials.get("pass")

        # BigQuery
        self.keyfile = credentials.get("keyfile")
        self.method = credentials.get("method")

        # Snowflake
        self.account = credentials.get("account")
        self.role = credentials.get("role")
        self.private_key_path = credentials.get("private_key_path")
        self.private_key_passphrase = credentials.get("private_key_passphrase")
        self.warehouse = credentials.get("warehouse")

        # Redshift
        self.host = credentials.get("host")
        self.port = credentials.get("port")

        self.engine = self.generate_engine()

    def generate_engine(self):
        engine = None
        if self.type == "bigquery":
            uri = f"bigquery://{self.database}/{self.schema}"
            if self.method == "oauth":
                engine = create_engine(uri)
            elif self.method == "service-account":
                engine = create_engine(uri, credentials_path=self.keyfile)
        elif self.type == "snowflake":
            if self.password:
                uri = f"snowflake://{self.user}:{self.password}@{self.account}/{self.warehouse}"
            engine = create_engine(uri)
        elif self.type in ["redshift", "postgres"]:
            uri = f"postgresql://{self.user}:{self.password}@{self.host}:{self.port}/{self.database}"
            engine = create_engine(uri)
        else:
            raise Exception(f"Database type {self.type} not supported.")
        if not engine:
            raise Exception(
                f"Authentication method for database type {self.type} not supported."
            )
        return engine

    def generate_query(self, schema, database):
        if self.type == "bigquery":
            query = f"select table_name from `{database}.{schema}.INFORMATION_SCHEMA.TABLES`"  # nosec
        elif self.type == "snowflake":
            query = f"select table_name from {database}.information_schema.tables where table_schema = '{schema.upper()}'"  # nosec
        elif self.type in ["redshift", "postgres"]:
            query = f"select table_name from information_schema.tables where table_schema = '{schema}'"  # nosec
        return query

    def run_query(self, schemas, database):
        tables = set()
        for schema in schemas:
            query = self.generate_query(schema, database)
            rows = self.engine.execute(query).fetchall()
            for row in rows:
                tables.add(row[0].lower())
        return tables



import click
from autogen.runner import Runner
from autogen.database import Warehouse
from autogen.config import Config
import ruamel.yaml
from pathlib import Path


def main():

    # Setup ruamel.yaml
    yml = ruamel.yaml.YAML()
    yml.indent(mapping=2, sequence=4, offset=2)
    yml.preserve_quotes = True

    # Get API key and secret
    api_key = click.prompt("Enter your Fivetran API key", type=str)
    api_secret = click.prompt("Enter your Fivetran API secret", type=str)

    # Create a runner
    runner = Runner(api_key, api_secret)

    # Confirm which group to use
    group_id = get_group_id(runner)

    # Confirm which connector type to use
    connector_selection = get_connector_type(runner, group_id)

    # Get connectors
    connectors = runner.get_connectors(group_id, connector_selection)

    # TODO: change this to work with multiple connectors
    schemas = list(connectors.keys())
    # connector_ids = list(connectors.values())

    # Load config
    config = Config(connector_selection)

    # Update packages.yml
    update_packages(config.name, config.version, yml)

    # Update dbt_project.yml
    update_dbt_project(
        config,
        schemas,
        yml,
    )


def get_group_id(runner):
    groups = runner.get_groups()
    groups_option_list = list(groups.keys())
    if len(groups_option_list) == 1:
        group_id = groups[groups_option_list[0]]
    else:
        print("\nAvailable groups:")
        groups_prompt_msg = (
            "\n".join([f"[{n+1}] {v}" for n, v in enumerate(groups_option_list)])
            + "\nDesired option (enter a number)"
        )
        numeric_group_selection = click.prompt(groups_prompt_msg, type=int)
        group_id = groups[groups_option_list[numeric_group_selection - 1]]
    return group_id


def get_connector_type(runner, group_id):
    print("\nAvailable connectors:")
    connector_types = runner.get_connector_types(group_id)
    connector_prompt_msg = (
        "\n".join([f"[{n+1}] {v}" for n, v in enumerate(connector_types)])
        + "\nDesired option (enter a number)"
    )
    numeric_connector_selection = click.prompt(connector_prompt_msg, type=int)
    connector_selection = connector_types[numeric_connector_selection - 1]
    return connector_selection


def update_packages(package_name, package_version, yml):

    # Create file it not exists
    file = open("packages.yml", "a+").close()

    # TODO: This should be doable in one pass, but having issue with opening modes
    with open("packages.yml", "r") as file:
        packages = yml.load(file)

    # If no packages, create full structure
    if packages is None:
        packages = {"packages": [{"package": package_name, "version": package_version}]}

    # If package is declared already, update version
    elif any([p for p in packages["packages"] if p["package"] == package_name]):
        package = [p for p in packages["packages"] if p["package"] == package_name][0]
        package["version"] = package_version

    # Otherwise, append new package to packages
    else:
        packages["packages"].append(
            {"package": package_name, "version": package_version}
        )

    with open("packages.yml", "w") as file:
        yml.dump(packages, file)


def load_dbt_project(yml):
    with open("dbt_project.yml", "r") as file:
        project = yml.load(file)
    return project


def update_dbt_project(
    config,
    schemas,
    yml,
):
    project = load_dbt_project(yml)

    profile_name = project["profile"]
    dbt_profiles_path = Path(Path.home(), ".dbt", "profiles.yml")

    with open(dbt_profiles_path, "r") as file:
        profiles = yml.load(file)
        credentials = dict(
            profiles[profile_name]["outputs"][profiles[profile_name]["target"]]
        )

    if not project.get("vars"):
        project["vars"] = {}

    project["vars"][config.schema_variable] = schemas[0]
    database = click.prompt(
        f"\nEnter the database/project of your {config.connector}", type=str
    )
    project["vars"][config.database_variable] = database

    warehouse = Warehouse(credentials)
    connector_tables = warehouse.run_query(schemas[:1], database)

    for v in config.enabled_variables:
        if "required_tables" in v:
            enabled = all(item in connector_tables for item in v["required_tables"])
            project["vars"][v["name"]] = enabled

    for v in config.preset_variables:
        if v["type"] == "list":
            project["vars"][v["name"]] = []

    with open("dbt_project.yml", "w") as file:
        yml.dump(project, file)


if __name__ == "__main__":
    main()



from pathlib import Path
from setuptools import setup
from autogen import __version__

here = Path(__file__).parent.resolve()

with (here / "README.md").open(encoding="utf-8") as file:
    long_description = file.read()

setup(
    name="fivetran_autogen",
    version=__version__,
    py_modules=["fivetran_autogen"],
    packages=["autogen", "autogen.packages"],
    include_package_data=True,
    entry_points={"console_scripts": ["fivetran_autogen = autogen.cli:main"]},
    author="Fivetran",
    author_email="hello@fivetran.com",
    url="https://github.com/fivetran/dbt_autogen",
    install_requires=[
        "click==8.0.3",
        "google-cloud-bigquery==2.32.0",
        "google-cloud-bigquery-storage==2.11.0",
        "jsonschema==4.4.0",
        "pyarrow==6.0.1",
        "PyYAML==6.0",
        "requests==2.27.1",
        "ruamel.yaml==0.17.20",
        "snowflake-sqlalchemy==1.3.3",
        "SQLAlchemy==1.4.25",
        "sqlalchemy-bigquery==1.3.0",
    ],
    long_description=long_description,
    long_description_content_type="text/markdown",
)
